import gc

def update_instances(new_class):
    """A decorator that updates all existing instances of a class,
    when that class is redefined.

    NB: Only works for new-style classes."""

    # Get the already-existing class with the same name, or None
    old_class = globals().get(new_class.__name__)

    # If an old version of this class already exists
    if old_class:
        # Find all its instances
        for existing_instance in [ instance for instance in gc.get_objects() 
                                   if type(instance) is old_class ]:
            # And make them instances of the new class
            existing_instance.__class__ = new_class

    # Don't change the new class itself in any way
    return new_class

# Let's test it

# Create a simple class with some internal state and some metho.
@update_instances
class Foo(object):

    def __init__(self, n):
        self.n = n

    def doit(self, n):
        return self.n + n

# Create some instances
a,b,c,d = map(Foo, range(4))
# And call the method on each instance
for instance in (a,b,c,d):
    print instance.doit(10)

# Redefine the class
@update_instances
class Foo(object):

    def __init__(self, n):
        self.n = n

    def doit(self, n):
        # Changed + to * in the next line
        return self.n * n

# Check that the behaviour of the existing instances has changed

for instance in (a,b,c,d):
    print instance.doit(10)
